const percentString = '%'
const lang = $('body').hasClass('german');

$(document).ready(function () {
    const gameBoards = document.getElementsByClassName('dnd_game');

    if (gameBoards.length > 0) {
        for (let i = 0; i < gameBoards.length; i++) {
            const gameDataVarId = gameBoards[i].getAttribute("data-game-data-var");

            boardInit(gameBoards[i], gameDataVarId);
        }
    }
});

function boardInit(gameBoard, gameDataVarId) {
    const gameData = window[gameDataVarId]; // relevant game data

    let activeAnswer;

    const questionsDiv = $('<div id="questions" />'); // create droppable and draggable div; check button
    const answersDiv = $('<div id="answers" />');
    const checkButton = $('<button>', {
        class: 'check_button',
        html: lang ? '\u00DCberpr\u00FCfe' : 'Provjeri'
    }).click(function () {   // event listener
        handleClick(questionsDiv);
    });
    const resetButton = $('<button>', {
        class: 'check_button',
        html: lang ? 'Wiederhole' : 'Ponovi'
    }).click(reset);

    answersDiv.appendTo(gameBoard);
    questionsDiv.appendTo(gameBoard);
    checkButton.appendTo(gameBoard);
    resetButton.appendTo(gameBoard);

    $(gameBoard).on('pointerdown', function () {
        if (activeAnswer) {
            removeActive(activeAnswer);
        }
    });
    $(gameBoard).droppable({
        drop: function (e, ui) {
            revertElement(answersDiv, ui); // returns element to its original container
            setTimeout(function () { $(ui.draggable[0]).draggable('option', 'revert', false); }, 500); // timeout because no current droppable/draggable callbacks is good enough for this
        }
    });

    const background = gameData.background ? $('<img>', { class: 'background', src: gameData.background }).appendTo(questionsDiv) : $.noop(); // when using dropzones

    const questions = createQuestions(gameData.tasks);
    const answers = createAnswers(gameData.tasks);

    shuffle(questions);
    shuffle(answers);

    for (let i = 0; i < questions.length; i++) {
        questions[i].on('pointerdown', function (e) {
            e.stopPropagation();
            clickQuestion(this);
        });
        questions[i].droppable({
            tolerance: 'pointer',
            greedy: true,
            drop: function (e, ui) {
                e.stopPropagation();
                clickQuestion(e.target);
            }
        });

        answers[i].on('pointerdown', function (e) {
            e.stopPropagation();
            clickAnswer(this);
        });
        answers[i].draggable({
            containment: gameBoard,
        });
        answers[i].on('dragstop', function () {
            removeActive(activeAnswer);
        });

        questions[i].appendTo(questionsDiv);
        answers[i].appendTo(answersDiv);
    }

    function clickAnswer(answer) {
        if (answer.classList.contains('solved')) {
            return;
        }
        if (answer.classList.contains('invalid')) {
            answer.classList.remove('invalid');
        }

        if (activeAnswer) {
            if (activeAnswer === answer) {
                removeActive(answer);
                return;
            } else if (answer.classList.contains('paired')) {
                answer.classList.remove('paired');
                switchPos(activeAnswer, answer);
                removeActive(activeAnswer);
                return;
            } else {
                removeActive(activeAnswer);
            }
        }
        makeActive(answer);
    }

    function clickQuestion(question) {
        if (!activeAnswer) {
            return;
        }
        if ($(question).children().length) {
            switchPos($(question).children()[0], activeAnswer);
        } else {
            $(activeAnswer).detach();
            $(activeAnswer).appendTo(question);
            $(activeAnswer).position({
                my: 'center',
                at: 'center',
                of: question,
            });
            activeAnswer.classList.add('paired');
            removeActive(activeAnswer);
        }
    }

    function makeActive(el) {
        el.classList.add('highlighted');
        activeAnswer = el;
    }

    function removeActive(el) {
        if (!el) {
            return;
        }
        el.classList.remove('highlighted');
        activeAnswer = undefined;
    }

    function reset() {
        const currentBoardElements = $(gameBoard).children();
        for (let i = 0; i < currentBoardElements.length; i++) {
            currentBoardElements[i].remove();
        }
        boardInit(gameBoard, gameDataVarId);
    }
}

function createAnswers(tasks) {
    let answers = [];
    for (let i = 0; i < tasks.length; i++) {
        switch (tasks[i].answer.type) {
            case 'img': {
                break;
            }
            case 'text': {
                const answer = $('<p>', {
                    id: tasks[i].id,
                    class: 'answer',
                    html: tasks[i].answer.content
                });
                answers.push(answer);
                break;
            }
            case 'number': {
                const answer = $('<p>', {
                    id: tasks[i].id,
                    class: 'answer',
                    html: tasks[i].answer.content,
                    height: 10 + percentString,
                    width: 10 + percentString
                });
                answers.push(answer);
                break;
            }
            default: {
                const answer = $('<span>', {
                    id: tasks[i].id,
                    class: 'answer',
                    html: tasks[i].answer.content
                });
                answers.push(answer);
                break;
            }
        }
    }
    return answers;
}

function createQuestions(tasks, offset) {
    let questions = [];
    for (let i = 0; i < tasks.length; i++) {
        switch (tasks[i].question.type) {
            case 'img': {
                const question = $('<img>', {
                    class: 'question_img',
                    id: tasks[i].id,
                    src: tasks[i].question.content
                });
                questions.push(question);
                break;
            }
            case 'dropzone': {
                const question = $('<div>', {
                    id: tasks[i].id,
                    class: 'question_dropzone bordered'
                }).css({
                    left: tasks[i].question.x + percentString,
                    top: tasks[i].question.y + percentString,
                    height: tasks[i].question.height + percentString,
                    width: tasks[i].question.width + percentString
                });
                questions.push(question);
                break;
            }
            default: {
                const question = $('<p>', {
                    id: tasks[i].id,
                    html: tasks[i].question.content
                })
                questions.push(question);
                break;
            }
        }
    }
    return questions;
}

function isMatch(answer, question) {
    if (answer.getAttribute('id') === question.getAttribute('id')) {
        answer.classList.add('valid');
        $(answer).draggable({ disabled: true });
        return true;

    } else {
        answer.classList.add('invalid');
        return false
    }
}

function handleClick(qContainer) {
    const questions = qContainer.children('div');
    for (let i = 0; i < questions.length; i++) {
        const answer = $(questions[i]).find('span');
        if (answer.length) {
            if (!answer[0].classList.contains('valid')) {
                if (isMatch(answer[0], questions[i])) {
                    answer[0].classList.remove('paired');
                    answer[0].classList.add('solved');
                }
            }
        }
    }
}

function switchPos(el1, el2) {
    const parent1 = $(el1).parent();
    const parent2 = $(el2).parent();

    if (el1.classList.contains('invalid')) {
        el1.classList.remove('invalid');
    }

    if (el2.classList.contains('invalid')) {
        el2.classList.remove('invalid');
    }

    $(el1).detach();
    $(el2).detach();

    $(el1).css({
        top: 0,
        left: 0,
    });

    $(el2).css({
        top: 0,
        left: 0,
    });

    parent1.append(el2);
    parent2.append(el1);

    if (parent1[0].classList.contains('question_dropzone')) {
        el2.classList.add('paired');
        $(el2).position({
            my: 'center',
            at: 'center',
            of: parent1[0],
        });
    }

    if (parent2[0].classList.contains('question_dropzone')) {
        el1.classList.add('paired');
        $(el1).position({
            my: 'center',
            at: 'center',
            of: parent2[0],
        });
    }
}

function revertElement(originalContainer, ui) {
    const el = $(ui.draggable[0]);
    if (el.parent()[0] !== originalContainer[0]) {
        el.appendTo(originalContainer);
        el.css({
            top: 0,
            left: 0
        });
    } else {
        el.data('uiDraggable').originalPosition = {
            top: 0,
            left: 0
        };
        el.draggable({
            revert: true,
            revertDuration: 0,
        });
    }
}

function shuffle(a) {
    var j, x, i;
    for (i = a.length - 1; i > 0; i--) {
        j = Math.floor(Math.random() * (i + 1));
        x = a[i];
        a[i] = a[j];
        a[j] = x;
    }
    return a;
}